#!/bin/sh
#   Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

# source common functions
. ./errlib.sh
. ./jsonlib.sh
. ./util.sh

LOCKFILE="/var/lock/systemupdate"

NOW=$(date +"%Y-%m-%d_%H%M%S")
BACKUP_BASENAME="dss_backup_$NOW"
BACKUP_ARCHIVE="/tmp/$BACKUP_BASENAME.dss"

XML_FILE_LIST="$(find /usr/share/dss/data/savedprops/ -type f -name *.xml)"
XML_FILE_LIST="$XML_FILE_LIST /usr/share/dss/data/apartment.xml \
               /usr/share/dss/data/config.d/location.xml"
FILE_LIST="/etc/default/ntpclient /etc/timezone"
BACKUP_FILE_LIST=""
RUNIT_DOWN=0
RUNIT_UP=1
RUNIT_STATUS=2

json_backup()  {
    echo -n "{ \"success\": true, \"code\": $ERR_OK, \"message\": \"OK\", "
    echo    "\"data\": { \"backup\": \"$1\" } }"
}

add_to_backup_list() {
    if [[ -z "$BACKUP_FILE_LIST" ]]; then
        BACKUP_FILE_LIST="$1"
    else
        BACKUP_FILE_LIST="$BACKUP_FILE_LIST $1"
    fi
}

echo "Content-Type: application/json"
echo ""

if [[ ! -x $XMLLINT ]]; then
    json_error $ERR_DOWAIT "xmllint tool not found"
    exit 1
fi

if [[ -e $LOCKFILE ]]; then
    json_error $ERR_DOWAIT "Upate in progress, please try again later"
    exit 1
fi

touch $LOCKFILE

# remove old backup files
rm -f /tmp/*.dss >/dev/null 2>&1

control_dss $RUNIT_STATUS
DSS_STATUS=$?

control_dss $RUNIT_DOWN

log "Preparing backup file $BACKUP_ARCHIVE"

for file in $XML_FILE_LIST
do
    if [[ -f $file ]]; then
        check_xml_file $file
        if [[ $? -eq $ERR_OK ]]; then
            add_to_backup_list $file
        else
            log "Will not backup $file because XML is not well formed"
        fi
    fi
done

check_host_name
if [[ $? -eq $ERR_OK ]]; then
    add_to_backup_list "/etc/hostname"
else
    log "Will not backup hostname, because confugration file is invalid"
fi

check_ssh_config
if [[ $? -eq $ERR_OK ]]; then
    add_to_backup_list "/etc/default/dropbear"
else
    log "Will not backup ssh settings, because configuration is invalid"
fi

for file in $FILE_LIST
do
    if [[ -f $file ]]; then
        add_to_backup_list $file
    fi
done

if [[ -z "$BACKUP_FILE_LIST" ]]; then
    json_error $ERR_BACKUP "No valid files found for backup"
    log "No valid files found for backup"
    rm -f $LOCKFILE
    control_dss $DSS_STATUS
    exit 1
fi

OUT=$(tar -zcf $BACKUP_ARCHIVE $BACKUP_FILE_LIST 2>&1)
if [[ $? -ne $ERR_OK ]]; then
    json_error $ERR_BACKUP "Could not create backup archive"
    log "Could not create backup archive: $OUT"
    control_dss $DSS_STATUS
    rm -f $LOCKFILE
    exit 1
else
    log "List of backed up files: $BACKUP_FILE_LIST"
fi

json_backup "$BACKUP_BASENAME"
log "Backup created as $BACKUP_ARCHIVE and ready for download"

control_dss $DSS_STATUS
rm -f $LOCKFILE
